@@ -1,6 +1,6 @@
 /* Nessus Attack Scripting Language 
  *
- * Copyright (C) 2002 - 2003 Michel Arboi and Renaud Deraison
+ * Copyright (C) 2002 - 2004 Tenable Network Security
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2,
@@ -15,17 +15,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * In addition, as a special exception, Renaud Deraison and Michel Arboi
- * give permission to link the code of this program with any
- * version of the OpenSSL library which is distributed under a
- * license identical to that listed in the included COPYING.OpenSSL
- * file, and distribute linked combinations including the two.
- * You must obey the GNU General Public License in all respects
- * for all of the code used other than OpenSSL.  If you modify
- * this file, you may extend this exception to your version of the
- * file, but you are not obligated to do so.  If you do not wish to
- * do so, delete this exception statement from your version.
- *
  */
  /*
   * This file contains all the function that make the "glue" between
@@ -46,17 +35,24 @@
 #include "nasl_debug.h"
 #include "nasl_nessusd_glue.h"
 
+#ifndef NASL_DEBUG
+#define NASL_DEBUG 0
+#endif
 
 /*------------------- Private utilities ---------------------------------*/
 
 static int isalldigit(char * str, int len)
 {
  int i ;
+ char buf[1024];
  for (i = 0; i < len; i ++)
  {
   if(!isdigit(str[i]))return 0;
  }
- return 1;
+
+ snprintf(buf, sizeof(buf), "%d", atoi(str));
+ if ( strcmp(buf, str) != 0 ) return 0;
+ else return 1;
 }
 
 
@@ -405,6 +401,134 @@ tree_cell * script_get_preference(lex_ctxt * lexic)
 	 return FAKE_CELL;
 }
 
+tree_cell * script_get_preference_file_content(lex_ctxt * lexic)
+{
+ struct arglist *  script_infos = lexic->script_infos;
+ tree_cell * retc;
+ char * pref = get_str_var_by_num(lexic, 0);
+ char * value;
+ int fd, n;
+ struct stat st;
+ char * buffer;
+ 
+ /* 
+  * Only signed scripts can access files uploaded by the user 
+  */
+ if (check_authenticated(lexic) < 0)
+   {
+     nasl_perror(lexic, "script_get_preference_file_content: script is not authenticated!\n");
+     return NULL;
+   }
+
+ if(pref == NULL){
+ 	nasl_perror(lexic, "Argument error in the function script_get_preference()\n");
+	nasl_perror(lexic, "Function usage is : pref = script_get_preference_file_content(<name>)\n");
+ 	return NULL;
+	}
+
+ value = get_plugin_preference(script_infos, pref);
+ if(value == NULL) return NULL;
+
+ value = (char*)get_plugin_preference_fname(script_infos, value);
+ if ( value == NULL ) return FAKE_CELL;
+
+ fd = open(value, O_RDONLY);
+ if (fd < 0)
+   {
+     nasl_perror(lexic, "script_get_preference_file_content: open(%s): %s\n",
+		 value, strerror(errno));
+     return NULL;
+   }
+ 
+ if (fstat(fd, &st) < 0)
+   {
+     nasl_perror(lexic, "script_get_preference_file_content: fstat(%s): %s\n",
+		 value, strerror(errno));
+     return NULL;
+   }
+
+ buffer = emalloc ( st.st_size );
+ n = 0;
+ while ( n < (int)st.st_size )
+ {
+   int	e;
+   errno = 0;
+   e = read(fd, buffer + n , (int)st.st_size - n);
+   if (e > 0)
+     n+= e;
+   else if (e == 0)		/* EOF */
+   {
+     nasl_perror(lexic, "script_get_preference_file_content: unexpected EOF on %s\n", value);
+     break;
+   }
+   else				/* error */
+       if (errno == EINTR) continue;
+       else
+       {
+	 nasl_perror(lexic, "script_get_preference_file_content: read(%s): %s",
+		     value, strerror(errno));
+	 break;
+       }
+ }
+
+ close(fd);
+ 
+
+ retc = alloc_tree_cell(0, NULL);
+ retc->type = CONST_DATA; 
+ retc->size = n;
+ retc->x.str_val = buffer;
+
+ return retc;
+}
+
+
+tree_cell*
+script_get_preference_file_location(lex_ctxt* lexic)
+{
+  struct arglist	*script_infos = lexic->script_infos;
+  tree_cell		*retc;
+  char			*pref = get_str_var_by_num(lexic, 0);
+  const char		*value, *local;
+  int			len;
+ 
+ /* 
+  * Getting the local file name is not dangerous, but
+  * only signed scripts can access files uploaded by the user 
+  */
+ if (check_authenticated(lexic) < 0)
+   {
+     nasl_perror(lexic, "script_get_preference_file_location: script is not authenticated!\n");
+     return NULL;
+   }
+
+  if(pref == NULL)
+    {
+      nasl_perror(lexic, "script_get_preference_file_location: no preference name!\n");
+      return NULL;
+    }
+
+  value = get_plugin_preference(script_infos, pref);
+  if(value == NULL)
+    {
+      nasl_perror(lexic, "script_get_preference_file_location: could not get preference %s\n", pref);
+      return NULL;
+    }
+  local = get_plugin_preference_fname(script_infos, value);
+  if(local == NULL)
+    {
+      nasl_perror(lexic, "script_get_preference_file_location: could not get local file name from preference %s\n", pref);
+      return NULL;
+    }
+
+  len = strlen(local);
+  retc = alloc_typed_cell(CONST_DATA);
+  retc->size = len;
+  retc->x.str_val = emalloc(len+1);
+  memcpy(retc->x.str_val, local, len+1);
+
+  return retc;
+}
 
 /* Are safe checks enabled ? */
 tree_cell * safe_checks(lex_ctxt * lexic)
@@ -428,15 +552,17 @@ tree_cell * safe_checks(lex_ctxt * lexic)
 
 /*--------------------[ KB ]---------------------------------------*/
 
+#define SECRET_KB_PREFIX	"Secret/"
 
 tree_cell * get_kb_list(lex_ctxt * lexic)
 {
  struct arglist * script_infos = lexic->script_infos;
- struct arglist * kb = arg_get_value(script_infos, "key");	/* gruik gruik */
+ struct kb_item ** kb = plug_get_kb(script_infos);
  char * kb_mask = get_str_var_by_num(lexic, 0);
  tree_cell * retc;
  int num_elems = 0;
  nasl_array * a;
+ struct kb_item * res, * top;
  
  
  if(kb_mask == NULL)
@@ -453,54 +579,50 @@ tree_cell * get_kb_list(lex_ctxt * lexic)
  retc = alloc_tree_cell(0,NULL);
  retc->type = DYN_ARRAY;
  retc->x.ref_val = a = emalloc(sizeof(nasl_array));
- while(kb->next != NULL)
+ 
+ top = res = kb_item_get_pattern(kb, kb_mask);
+ 
+ while(res != NULL)
  {
   anon_nasl_var v;
   bzero(&v, sizeof(v));
-  if(str_match(kb->name, kb_mask, 1) != 0)
-  {
-  if(kb->type == ARG_INT)
+  
+  if (lexic->authenticated || 
+      strncmp(res->name, SECRET_KB_PREFIX, sizeof(SECRET_KB_PREFIX) - 1) != 0)
+    {
+  if(res->type == KB_TYPE_INT)
   {
   v.var_type = VAR2_INT;
-  v.v.v_int = (int)kb->value;
-  add_var_to_array(a, kb->name, &v);
+  v.v.v_int = res->v.v_int;
+  add_var_to_array(a, res->name, &v);
   num_elems ++;
   }
-  else if(kb->type == ARG_STRING)
+  else if(res->type == KB_TYPE_STR)
   {
    v.var_type = VAR2_DATA;
-   v.v.v_str.s_val = kb->value;
-   v.v.v_str.s_siz = strlen(kb->value);
-   add_var_to_array(a, kb->name, &v);
+   v.v.v_str.s_val = res->v.v_str;
+   v.v.v_str.s_siz = strlen(res->v.v_str);
+   add_var_to_array(a, res->name, &v);
    num_elems ++;
   }
-  else if(kb->type == ARG_ARGLIST)
-  {
-   struct arglist * tmp = kb->value;
-   if( tmp != NULL )
-     while (tmp->next != NULL )
-     {
-       v.var_type = VAR2_DATA;
-       v.v.v_str.s_val = tmp->value;
-       v.v.v_str.s_siz = strlen(tmp->value);
-       add_var_to_array(a, kb->name, &v);
-       num_elems ++;
-       tmp = tmp->next;
-     }
-   }
-  }
-  kb = kb->next;
+    }
+#if NASL_DEBUG > 0
+  else
+    nasl_perror(lexic, "get_kb_list: skipping protected KN entry %s\n", res->name);
+#endif
+  res = res->next;
  }
  
+ kb_item_get_all_free(top);
+ 
  if(num_elems == 0)
  {
   deref_cell(retc);
-  return NULL;
+  return FAKE_CELL;
  }
  return retc;
 }
 
-
 tree_cell * get_kb_item(lex_ctxt * lexic)
 {
  struct arglist * script_infos = lexic->script_infos;
@@ -508,53 +630,164 @@ tree_cell * get_kb_item(lex_ctxt * lexic)
  char * kb_entry = get_str_var_by_num(lexic, 0);
  char * val;
  tree_cell * retc;
+ int type;
 
  if(kb_entry == NULL)
 	 return NULL;
 
- val = plug_get_key(script_infos,kb_entry);
+ if (! lexic->authenticated && 
+     strncmp(kb_entry, SECRET_KB_PREFIX, sizeof(SECRET_KB_PREFIX) - 1) == 0)
+   {
+     nasl_perror(lexic, "Untrusted script cannot read protected KB entry %s\n",
+		 kb_entry);
+     return NULL;
+   }
+   
+ val = plug_get_key(script_infos,kb_entry, &type);
 
 
+ if ( val == NULL && type == -1 )
+  return NULL;
+ 
+
  retc = alloc_tree_cell(0, NULL);
- if ( val == NULL )
- {
-  /* Shall we return a const int of 0 ? */
-  retc->type = CONST_DATA;	
-  retc->size = 0;
-  retc->x.str_val = estrdup("");
-  return retc;
- }
- 
- if(isalldigit(val, strlen(val)))
+ if( type == KB_TYPE_INT )
  {
   retc->type = CONST_INT;
-  retc->x.i_val = atoi(val);
+  retc->x.i_val = (int)val;
   return retc;
  }
  else
  {
  retc->type = CONST_DATA;
- retc->size = strlen(val);
- retc->x.str_val = estrdup(val);
+ if ( val != NULL )
+  {
+   retc->size = strlen(val);
+   retc->x.str_val = estrdup(val);
+  }
+ else {
+    retc->size = 0;
+    retc->x.str_val = NULL;
+   }
  }
 
  return retc;
 }
 
+/* 
+ * Instead of reading the local copy of the KB, we ask the upstream
+ * father the "newest" value of a given KB item. This is especially useful
+ * when dealing with shared sockets and SSH
+ */
+tree_cell * get_kb_fresh_item(lex_ctxt * lexic)
+{
+ struct arglist * script_infos = lexic->script_infos;
+
+ char * kb_entry = get_str_var_by_num(lexic, 0);
+ char * val;
+ tree_cell * retc;
+ int type;
+
+ if(kb_entry == NULL)
+	 return NULL;
+
+ if (! lexic->authenticated && 
+     strncmp(kb_entry, SECRET_KB_PREFIX, sizeof(SECRET_KB_PREFIX) - 1) == 0)
+   {
+     nasl_perror(lexic, "Untrusted script cannot read protected KB entry %s\n",
+		 kb_entry);
+     return NULL;
+   }
+   
+ val = plug_get_fresh_key(script_infos,kb_entry, &type);
+
+
+ if ( val == NULL && type == -1 )
+  return NULL;
+ 
+
+ retc = alloc_tree_cell(0, NULL);
+ if( type == KB_TYPE_INT )
+ {
+  retc->type = CONST_INT;
+  retc->x.i_val = (int)val;
+  return retc;
+ }
+ else
+ {
+ retc->type = CONST_DATA;
+ if ( val != NULL )
+  {
+   retc->size = strlen(val);
+   retc->x.str_val = val; /* Do NOT estrdup() the value, since plug_get_fresh_key() allocated the memory already */
+  }
+ else {
+    retc->size = 0;
+    retc->x.str_val = NULL;
+   }
+ }
+
+ return retc;
+}
+
+tree_cell * replace_kb_item(lex_ctxt * lexic)
+{
+ struct arglist * script_infos = lexic->script_infos;
+ char * name  = get_str_local_var_by_name(lexic, "name");
+ int type = get_local_var_type_by_name(lexic, "value");
+ 
+ if( name == NULL )
+ {
+  nasl_perror(lexic, "Syntax error with replace_kb_item() [null name]\n", name);
+  return FAKE_CELL;
+ }
+ 
+ if(type == VAR2_INT)
+ {
+  int value = get_int_local_var_by_name(lexic, "value", -1);
+  if ( value != -1 )plug_replace_key(script_infos, name, ARG_INT,(void*)value);
+  else nasl_perror(lexic, "Syntax error with replace_kb_item(%s) [value=-1]\n", name); }
+ else
+ {
+  char * value = get_str_local_var_by_name(lexic, "value");
+  if( value == NULL )
+  {
+    nasl_perror(lexic, "Syntax error with replace_kb_item(%s) [null value]\n", name);
+    return FAKE_CELL;
+  }
+  plug_replace_key(script_infos, name, ARG_STRING, value);
+ }
+
+ return FAKE_CELL;
+}
+
 tree_cell * set_kb_item(lex_ctxt * lexic)
 {
  struct arglist * script_infos = lexic->script_infos;
- char * name = get_str_local_var_by_name(lexic, "name");
- char * value= get_str_local_var_by_name(lexic, "value");
-
+ char * name  = get_str_local_var_by_name(lexic, "name");
+ int type = get_local_var_type_by_name(lexic, "value");
  
- if(name != NULL && value != NULL)
-  plug_set_key(script_infos, name, ARG_STRING, value);
+ if( name == NULL )
+ {
+  nasl_perror(lexic, "Syntax error with set_kb_item() [null name]\n", name);
+  return FAKE_CELL;
+ }
+ 
+ if(type == VAR2_INT)
+ {
+  int value = get_int_local_var_by_name(lexic, "value", -1);
+  if ( value != -1 )plug_set_key(script_infos, name, ARG_INT,(void*)value);
+  else nasl_perror(lexic, "Syntax error with set_kb_item() [value=-1]\n", name); }
  else
-   {
-     nasl_perror(lexic, "Syntax error with set_kb_item(name:0x%.2x, value: 0x%.2x)\n", name, value);
-     return NULL;
-   }
+ {
+  char * value = get_str_local_var_by_name(lexic, "value");
+  if( value == NULL )
+  {
+    nasl_perror(lexic, "Syntax error with set_kb_item() [null value]\n", name);
+    return FAKE_CELL;
+  }
+  plug_set_key(script_infos, name, ARG_STRING, value);
+ }
 
  return FAKE_CELL;
 }
@@ -584,7 +817,7 @@ static tree_cell * security_something(lex_ctxt * lexic, proto_post_something_t p
   int len = get_local_var_size_by_name(lexic, "data");
   int i;
   
-  dup = strndup(data, len);
+  dup = nasl_strndup(data, len);
   for(i=0;i<len;i++)
    if(dup[i] == 0)dup[i]=' ';
  }
@@ -639,6 +872,35 @@ tree_cell * security_note(lex_ctxt * lexic)
  return security_something(lexic, proto_post_note, post_note);
 }
 
+tree_cell * nasl_get_preference(lex_ctxt * lexic)
+{
+  tree_cell	*retc;
+  char		*name, *value;
+  struct arglist *script_infos, *prefs;
+
+
+  script_infos = lexic->script_infos;
+  prefs = arg_get_value(script_infos, "preferences");
+  if (prefs == NULL)
+    {
+      nasl_perror(lexic, "get_preference: not preferences\n");
+      return NULL;
+    }
+  name =  get_str_var_by_num(lexic, 0);
+  if (name == NULL)
+    {
+      nasl_perror(lexic, "get_preference: no name\n");
+      return NULL;
+    }
+  value = arg_get_value(prefs, name);
+  if (value == NULL)
+    return NULL;
+
+  retc = alloc_typed_cell(CONST_DATA);
+  retc->x.str_val = strdup(value);
+  retc->size = strlen(value);
+  return retc;
+}
 
 /*-------------------------[ Reporting an open port ]---------------------*/
 
@@ -708,16 +970,106 @@ tree_cell * nasl_scanner_add_port(lex_ctxt * lexic)
  
 tree_cell * nasl_scanner_status(lex_ctxt * lexic)
 {
- int current = get_int_local_var_by_name(lexic, "current", -1);
- int total   = get_int_local_var_by_name(lexic, "total", -1); 
- struct arglist * script_infos = lexic->script_infos;
- struct arglist * hostdata = arg_get_value(script_infos, "HOSTNAME");
+  int current = get_int_local_var_by_name(lexic, "current", -1);
+  int total   = get_int_local_var_by_name(lexic, "total", -1); 
+  struct arglist * script_infos = lexic->script_infos;
+  struct arglist * hostdata = arg_get_value(script_infos, "HOSTNAME");
 
- if(current != -1 && total != -1)
- {
-  struct arglist * globs = arg_get_value(script_infos, "globals");			
-  comm_send_status(globs, arg_get_value(hostdata, "FQDN"), "portscan",
-  		     current, total);
-  }
- return FAKE_CELL;
+  if(current != -1 && total != -1)
+    {
+      struct arglist * globs = arg_get_value(script_infos, "globals");
+      if (globs == NULL) return NULL;
+      comm_send_status(globs, arg_get_value(hostdata, "NAME"), "portscan", current, total);
+    }
+  return FAKE_CELL;
 } 
+
+
+
+/*--------------------[ SHARED SOCKETS ]---------------------------------------*/
+
+#define SECRET_SOCKET_PREFIX "Secret/"
+
+tree_cell * nasl_shared_socket_register( lex_ctxt * lexic )
+{
+  char * name = get_str_local_var_by_name(lexic, "name");
+  int    soc  = get_int_local_var_by_name(lexic, "socket", -1);
+  struct arglist * script_infos = lexic->script_infos;
+  int type, opt_len = sizeof(type);
+
+ if ( name == NULL || soc < 0 )
+ { 
+  fprintf(stderr, "Usage: shared_socket_register(name:<name>, socket:<soc>)\n");
+  return NULL;
+ }
+
+ if ( strncmp(name, SECRET_SOCKET_PREFIX, strlen(SECRET_SOCKET_PREFIX)) == 0 &&
+      check_authenticated(lexic) < 0 ) return NULL;
+
+
+ shared_socket_register(script_infos, soc, name);
+ return FAKE_CELL;
+}
+
+tree_cell * nasl_shared_socket_acquire( lex_ctxt * lexic )
+{
+  char * name = get_str_var_by_num(lexic, 0);
+  int fd;
+  tree_cell * retc;
+  struct arglist * script_infos = lexic->script_infos;
+
+ if ( name == NULL )
+ { 
+ fprintf(stderr, "Usage: shared_socket_acquire(<name>)\n");
+ return NULL;
+ }
+
+ if ( strncmp(name, SECRET_SOCKET_PREFIX, strlen(SECRET_SOCKET_PREFIX)) == 0 &&
+      check_authenticated(lexic) < 0 ) return NULL;
+
+ fd = shared_socket_acquire(script_infos, name);
+ if ( fd < 0 ) return NULL;
+ retc = alloc_tree_cell(0, NULL);
+ retc->type = CONST_INT;
+ retc->x.i_val = fd;
+ return retc;
+}
+
+tree_cell * nasl_shared_socket_release( lex_ctxt * lexic )
+{
+  char * name = get_str_var_by_num(lexic, 0);
+  int fd;
+  struct arglist * script_infos = lexic->script_infos;
+
+ if ( name == NULL )
+ { 
+ fprintf(stderr, "Usage: shared_socket_release(<name>)\n");
+ return NULL;
+ }
+
+ if ( strncmp(name, SECRET_SOCKET_PREFIX, strlen(SECRET_SOCKET_PREFIX)) == 0 &&
+      check_authenticated(lexic) < 0 ) return NULL;
+
+ shared_socket_release(script_infos, name);
+ return NULL;
+}
+
+tree_cell * nasl_shared_socket_destroy( lex_ctxt * lexic )
+{
+  char * name = get_str_var_by_num(lexic, 0);
+  int fd;
+  struct arglist * script_infos = lexic->script_infos;
+
+ if ( name == NULL )
+ { 
+ fprintf(stderr, "Usage: shared_socket_release(<name>)\n");
+ return NULL;
+ }
+
+ if ( strncmp(name, SECRET_SOCKET_PREFIX, strlen(SECRET_SOCKET_PREFIX)) == 0 &&
+      check_authenticated(lexic) < 0 ) return NULL;
+
+
+ shared_socket_destroy(script_infos, name);
+ return NULL;
+}
