@@ -1,6 +1,6 @@
 /* Nessus Attack Scripting Language 
  *
- * Copyright (C) 2002 - 2003 Michel Arboi and Renaud Deraison
+ * Copyright (C) 2002 - 2004 Tenable Network Security
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2,
@@ -15,17 +15,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * In addition, as a special exception, Renaud Deraison and Michel Arboi
- * give permission to link the code of this program with any
- * version of the OpenSSL library which is distributed under a
- * license identical to that listed in the included COPYING.OpenSSL
- * file, and distribute linked combinations including the two.
- * You must obey the GNU General Public License in all respects
- * for all of the code used other than OpenSSL.  If you modify
- * this file, you may extend this exception to your version of the
- * file, but you are not obligated to do so.  If you do not wish to
- * do so, delete this exception statement from your version.
- *
  */
 #include <includes.h>
 #include <nasl_raw.h>
@@ -110,21 +99,25 @@ tree_cell* forge_ip_packet(lex_ctxt* lexic)
 
   pkt->ip_len = FIX(20 + data_len);
   
-  pkt->ip_id = get_int_local_var_by_name(lexic, "ip_id", rand());
+  pkt->ip_id = htons(get_int_local_var_by_name(lexic, "ip_id", rand()));
   pkt->ip_off = get_int_local_var_by_name(lexic, "ip_off", 0);
   pkt->ip_off = FIX(pkt->ip_off);
   pkt->ip_ttl = get_int_local_var_by_name(lexic, "ip_ttl", 64);
   pkt->ip_p = get_int_local_var_by_name(lexic, "ip_p", 0);
-  pkt->ip_sum = get_int_local_var_by_name(lexic, "ip_sum", 0);
+  pkt->ip_sum = htons(get_int_local_var_by_name(lexic, "ip_sum", 0));
   /* source */
   s = get_str_local_var_by_name(lexic, "ip_src");
   if (s != NULL)
     inet_aton(s, &pkt->ip_src);
   /* else this host address? */
 
-
-    
-  pkt->ip_dst.s_addr = dst_addr->s_addr; 
+  /* I know that this feature looks dangerous, but anybody can edit an IP 
+   * packet with the string functions */
+  s = get_str_local_var_by_name(lexic, "ip_dst");
+  if (s != NULL)
+    inet_aton(s, &pkt->ip_dst);
+  else
+    pkt->ip_dst.s_addr = dst_addr->s_addr; 
   
   if( data != NULL )
   {
@@ -165,14 +158,14 @@ tree_cell * get_ip_element(lex_ctxt * lexic)
   }
   
   if(!strcmp(element, "ip_v"))		{ ret_int = ip->ip_v; flag ++;		}
-  else if(!strcmp(element, "ip_id"))	{ ret_int = ip->ip_id;  flag ++ ;	}
+  else if(!strcmp(element, "ip_id"))	{ ret_int = UNFIX(ip->ip_id);  flag ++ ;	}
   else if(!strcmp(element, "ip_hl"))	{ ret_int = ip->ip_hl;  flag ++;	}
   else if(!strcmp(element, "ip_tos"))	{ ret_int = ip->ip_tos; flag ++;	}
   else if(!strcmp(element, "ip_len"))	{ ret_int = UNFIX(ip->ip_len); flag ++;	}
   else if(!strcmp(element, "ip_off"))	{ ret_int = UNFIX(ip->ip_off); flag ++; }
   else if(!strcmp(element, "ip_ttl"))	{ ret_int = ip->ip_ttl; flag ++;	}
   else if(!strcmp(element, "ip_p"))	{ ret_int = ip->ip_p; flag ++;		}
-  else if(!strcmp(element, "ip_sum"))	{ ret_int = ip->ip_sum; flag ++;	}
+  else if(!strcmp(element, "ip_sum"))	{ ret_int = UNFIX(ip->ip_sum); flag ++;	}
   
   if(flag != 0)
   {
@@ -226,7 +219,7 @@ tree_cell * set_ip_elements(lex_ctxt * lexic)
  pkt->ip_v  = get_int_local_var_by_name(lexic, "ip_v",  pkt->ip_v);
  pkt->ip_tos= get_int_local_var_by_name(lexic, "ip_tos", pkt->ip_tos);
  pkt->ip_len= FIX(get_int_local_var_by_name(lexic, "ip_len", UNFIX(pkt->ip_len)));
- pkt->ip_id = get_int_local_var_by_name(lexic, "ip_id", pkt->ip_id);
+ pkt->ip_id = htons(get_int_local_var_by_name(lexic, "ip_id", pkt->ip_id));
  pkt->ip_off= FIX(get_int_local_var_by_name(lexic, "ip_off", UNFIX(pkt->ip_off)));
  pkt->ip_ttl= get_int_local_var_by_name(lexic, "ip_ttl", pkt->ip_ttl);
  pkt->ip_p  = get_int_local_var_by_name(lexic, "ip_p", pkt->ip_p);
@@ -235,7 +228,7 @@ tree_cell * set_ip_elements(lex_ctxt * lexic)
  if (s != NULL)
     inet_aton(s, &pkt->ip_src);
  
-  pkt->ip_sum = get_int_local_var_by_name(lexic, "ip_sum", 0);
+  pkt->ip_sum = htons(get_int_local_var_by_name(lexic, "ip_sum", 0));
   if(pkt->ip_sum == 0) 
     pkt->ip_sum = np_in_cksum((u_short *)pkt, sizeof(struct ip));
   
@@ -274,9 +267,10 @@ tree_cell * insert_ip_options(lex_ctxt * lexic)
  }
  
  pad_len = 4 - ((sizeof(uc_code) + sizeof(uc_len) + value_size) % 4);
+ if ( pad_len == 4 ) pad_len = 0;
  
  hl =  ip->ip_hl * 4 < UNFIX(ip->ip_len) ? ip->ip_hl * 4 : UNFIX(ip->ip_len);
- new_packet = emalloc(UNFIX(ip->ip_len) + 4 + value_size + pad_len);
+ new_packet = emalloc(size + 4 + value_size + pad_len);
  bcopy(ip, new_packet, hl);
  
  uc_code = (u_char)code;
@@ -290,7 +284,6 @@ tree_cell * insert_ip_options(lex_ctxt * lexic)
  
  
  
- pad_len = 4 - ((sizeof(uc_code) + sizeof(uc_len) + value_size) % 4);
  zero = 0;
  for(i=0;i<pad_len;i++)
  {
@@ -299,11 +292,11 @@ tree_cell * insert_ip_options(lex_ctxt * lexic)
  
 
  p = (char*)ip;
- bcopy(p + hl, new_packet + (sizeof(uc_code) + sizeof(uc_len) + value_size + pad_len) + hl, UNFIX(ip->ip_len) - hl);
+ bcopy(p + hl, new_packet + (sizeof(uc_code) + sizeof(uc_len) + value_size + pad_len) + hl, size - hl);
  
  
  new_packet->ip_hl  = (hl + (sizeof(uc_code) + sizeof(uc_len) + value_size + pad_len)) / 4;
- new_packet->ip_len = FIX(UNFIX(ip->ip_len) + sizeof(uc_code) + sizeof(uc_len) + value_size + pad_len);
+ new_packet->ip_len = FIX(size + sizeof(uc_code) + sizeof(uc_len) + value_size + pad_len);
  new_packet->ip_sum = 0;
  new_packet->ip_sum = np_in_cksum((u_short*)new_packet, new_packet->ip_hl*4 > UNFIX(new_packet->ip_len) ? UNFIX(new_packet->ip_len):new_packet->ip_hl*4  );
  
@@ -333,7 +326,7 @@ tree_cell * dump_ip_packet(lex_ctxt * lexic)
     printf("\tip_v  : %d\n", ip->ip_v);
     printf("\tip_tos: %d\n", ip->ip_tos);
     printf("\tip_len: %d\n", UNFIX(ip->ip_len));
-    printf("\tip_id : %d\n", ip->ip_id);
+    printf("\tip_id : %d\n", ntohs(ip->ip_id));
     printf("\tip_off: %d\n", UNFIX(ip->ip_off));
     printf("\tip_ttl: %d\n", ip->ip_ttl);
     switch(ip->ip_p)
@@ -348,7 +341,7 @@ tree_cell * dump_ip_packet(lex_ctxt * lexic)
      			printf("\tip_p  : %d\n", ip->ip_p);	
 			break;
      }					
-    printf("\tip_sum: 0x%x\n", ip->ip_sum);
+    printf("\tip_sum: 0x%x\n", ntohs(ip->ip_sum));
     printf("\tip_src: %s\n", inet_ntoa(ip->ip_src));
     printf("\tip_dst: %s\n", inet_ntoa(ip->ip_dst));
     printf("\n");
@@ -434,7 +427,7 @@ tree_cell*	forge_tcp_packet(lex_ctxt* lexic)
   if(!tcp->th_sum)
     {
       struct pseudohdr pseudoheader;
-      char * tcpsumdata = emalloc(sizeof(struct pseudohdr) + (len % 2 ? len + 1 : len) );
+      char * tcpsumdata = emalloc(sizeof(struct pseudohdr) + (len % 2 ? len + 1 : len ) );
       struct in_addr source, dest;
 
       source.s_addr = ip->ip_src.s_addr;
@@ -983,12 +976,12 @@ tree_cell*	forge_icmp_packet(lex_ctxt* lexic)
       if(ip->ip_hl * 4 > ip_sz)
       	return NULL;
 	
-      pkt = emalloc(sizeof(struct icmp) + ip->ip_hl * 4 + len);
+      pkt = emalloc(sizeof(struct icmp) + ip_sz + len);
       ip_icmp = (struct ip*) pkt;
   
       
   
-      bcopy(ip, ip_icmp, UNFIX(ip->ip_len));
+      bcopy(ip, ip_icmp, ip_sz);
       if(UNFIX(ip_icmp->ip_len) <= (ip_icmp->ip_hl * 4))
 	{
 	  if (get_int_local_var_by_name(lexic, "update_ip_len", 1) != 0)
@@ -1005,12 +998,14 @@ tree_cell*	forge_icmp_packet(lex_ctxt* lexic)
       icmp->icmp_type = t;
       icmp->icmp_seq = htons(get_int_local_var_by_name(lexic, "icmp_seq", 0));
       icmp->icmp_id = htons(get_int_local_var_by_name(lexic, "icmp_id", 0));
-      icmp->icmp_cksum = htons(get_int_local_var_by_name(lexic, "icmp_cksum", 0));
      
       if(data != NULL)bcopy(data, &(p[8]), len);
   
-      if(icmp->icmp_cksum == 0)
+      if(get_int_local_var_by_name(lexic, "icmp_cksum", -1) == -1 )
       	icmp->icmp_cksum = np_in_cksum((u_short *) icmp, len + 8);
+      else
+        icmp->icmp_cksum = htons(get_int_local_var_by_name(lexic, "icmp_cksum", 0));
+       
       
       retc = alloc_tree_cell(0, NULL);
       retc->type = CONST_DATA;
@@ -1044,12 +1039,19 @@ tree_cell * get_icmp_element(lex_ctxt * lexic)
   if(!strcmp(elem, "icmp_id"))value = ntohs(icmp->icmp_id);
   else if(!strcmp(elem, "icmp_code"))value = icmp->icmp_code;
   else if(!strcmp(elem, "icmp_type"))value = icmp->icmp_type;
+  else if(!strcmp(elem, "icmp_seq"))value = ntohs(icmp->icmp_seq);
+  else if(!strcmp(elem, "icmp_cksum"))value = ntohs(icmp->icmp_cksum);
   else if(!strcmp(elem, "data"))
   {
    retc = alloc_tree_cell(0, NULL);
    retc->type = CONST_DATA;
-   retc->size = UNFIX(ip->ip_len) - (ip->ip_hl * 4) - 8;
-   retc->x.str_val = strndup( &(p[ip->ip_hl*4+8]), retc->size);
+   retc->size = get_var_size_by_name(lexic, "icmp") - (ip->ip_hl * 4) - 8;
+   if ( retc->size > 0 )
+     retc->x.str_val = nasl_strndup( &(p[ip->ip_hl*4+8]), retc->size);
+   else {
+     retc->x.str_val = NULL;
+     retc->size = 0;
+   }
    return retc;
   }
   else return NULL;
@@ -1157,7 +1159,6 @@ tree_cell * nasl_tcp_ping(lex_ctxt * lexic)
  struct arglist *  script_infos = lexic->script_infos;
  struct in_addr * dst = plug_get_host_ip(script_infos);
  struct in_addr src;
- struct in_addr * ipa;
  struct sockaddr_in soca;
  int flag = 0;
  int i = 0;
@@ -1182,12 +1183,14 @@ tree_cell * nasl_tcp_ping(lex_ctxt * lexic)
  if( dst == NULL ) 
   	return NULL;
 	
- 
  for(i=0;i < sizeof(sports) / sizeof(int); i ++)
  {
   if ( sports[i] == 0 ) sports[i] = rnd_tcp_port();
  }
  
+
+ 
+ 
  for(i=0;ports[i];i++)num_ports ++;
  i = 0;
  
@@ -1206,16 +1209,12 @@ tree_cell * nasl_tcp_ping(lex_ctxt * lexic)
  if(port == -1)
 	 port = plug_get_host_open_port(script_infos);
  
- if(script_infos && (ipa = plug_get_key(script_infos, "localhost/ip")) != NULL)
-   src.s_addr = ipa->s_addr;
- else 
- {
-  if(islocalhost(dst) > 0 )src.s_addr = dst->s_addr;
-  else {
+ if(islocalhost(dst) > 0 )src.s_addr = dst->s_addr;
+  else 
+   {
     bzero(&src, sizeof(src));
     routethrough(dst, &src);
-  }
- }
+   }
  
  snprintf(filter, sizeof(filter), "ip and src host %s", inet_ntoa(*dst));
  bpf = init_capture_device(*dst, src, filter);
@@ -1359,11 +1358,11 @@ tree_cell*	nasl_send_packet(lex_ctxt* lexic)
 	{
 	  if(islocalhost(&sip->ip_dst))
 	    {
-	      answer = (u_char*) capture_next_packet(bpf, 5, &answer_sz);
+	      answer = (u_char*) capture_next_packet(bpf, to, &answer_sz);
 	      while(answer != NULL && (!memcmp(answer, (char*)ip, sizeof(struct ip))))
 		{
 		  efree(&answer);
-		  answer = (u_char*)capture_next_packet( bpf, 5, &answer_sz);
+		  answer = (u_char*)capture_next_packet( bpf, to, &answer_sz);
 		}
 	      	
 	    }
@@ -1384,7 +1383,6 @@ tree_cell*	nasl_send_packet(lex_ctxt* lexic)
   close(soc);
   return retc;
 }
-
    
    
 /*---------------------------------------------------------------------------*/
